use Test::More tests => 1;

BEGIN {
use_ok( 'Class::DBI::Plugin::AutoUntaint' );
}

diag( "Testing Class::DBI::Plugin::AutoUntaint $Class::DBI::Plugin::AutoUntaint::VERSION, Perl 5.008006, /usr/local/bin/perl" );
